@@ -60,38 +60,15 @@ from bigml.util import (invert_dictionary, localize, is_url, check_dir,
                         clear_console_line, reset_console_line, console_log,
                         maybe_save, get_exponential_wait)
 from bigml.util import DEFAULT_LOCALE
+from bigml.domain import Domain
+from bigml.domain import DEFAULT_DOMAIN, DEFAULT_PROTOCOL
 
 register_openers()
 
-# Base Domain
-BIGML_DOMAIN = os.environ.get('BIGML_DOMAIN', 'bigml.io')
-
-# Domain for prediction server
-BIGML_PREDICTION_DOMAIN = os.environ.get('BIGML_PREDICTION_DOMAIN',
-                                         BIGML_DOMAIN)
-# Whether a prediction server is being used or not
-PREDICTION_SERVER_ON = (BIGML_DOMAIN != BIGML_PREDICTION_DOMAIN)
-
-# Protocol for prediction server
-BIGML_PREDICTION_PROTOCOL = os.environ.get('BIGML_PREDICTION_PROTOCOL',
-                                           'https')
-
-# Check BigML.io host’s SSL certificate
-# DO NOT CHANGE IT.
-VERIFY = (BIGML_DOMAIN == "bigml.io")
-
-# Check prediction server's SSL certificate
-# DO NOT CHANGE IT.
-VERIFY_PREDICTION_SERVER = (BIGML_PREDICTION_DOMAIN == "bigml.com")
-
 # Base URL
-BIGML_URL = 'https://%s/andromeda/' % BIGML_DOMAIN
+BIGML_URL = '%s://%s/andromeda/'
 # Development Mode URL
-BIGML_DEV_URL = 'https://%s/dev/andromeda/' % BIGML_DOMAIN
-
-# Prediction URL
-BIGML_PREDICTION_URL = '%s://%s/andromeda/' % (BIGML_PREDICTION_PROTOCOL,
-                                               BIGML_PREDICTION_DOMAIN)
+BIGML_DEV_URL = '%s://%s/dev/andromeda/'
 
 # Basic resources
 SOURCE_PATH = 'source'
@@ -406,60 +383,6 @@ def exception_on_error(resource):
         raise Exception(resource['error']['status']['message'])
 
 
-def error_message(resource, resource_type='resource', method=None):
-    """Error message for each type of resource
-
-    """
-    error = None
-    error_info = None
-    if isinstance(resource, dict):
-        if 'error' in resource:
-            error_info = resource['error']
-        elif ('code' in resource
-              and 'status' in resource):
-            error_info = resource
-    if error_info is not None and 'code' in error_info:
-        code = error_info['code']
-        if ('status' in error_info and
-                'message' in error_info['status']):
-            error = error_info['status']['message']
-            extra = error_info['status'].get('extra', None)
-            if extra is not None:
-                error += ": %s" % extra
-        if code == HTTP_NOT_FOUND and method == 'get':
-            alternate_message = ''
-            if BIGML_DOMAIN != 'bigml.io':
-                alternate_message = (
-                    u'- The %s was not created in %s.\n' % (
-                        resource_type, BIGML_DOMAIN))
-            error += (
-                u'\nCouldn\'t find a %s matching the given'
-                u' id. The most probable causes are:\n\n%s'
-                u'- A typo in the %s\'s id.\n'
-                u'- The %s id cannot be accessed with your credentials.\n'
-                u'\nDouble-check your %s and'
-                u' credentials info and retry.' % (
-                    resource_type, alternate_message, resource_type,
-                    resource_type, resource_type))
-            return error
-        if code == HTTP_UNAUTHORIZED:
-            error += u'\nDouble-check your credentials, please.'
-            return error
-        if code == HTTP_BAD_REQUEST:
-            error += u'\nDouble-check the arguments for the call, please.'
-            return error
-        if code == HTTP_TOO_MANY_REQUESTS:
-            error += (u'\nToo many requests. Please stop '
-                      u' requests for a while before resuming.')
-            return error
-        elif code == HTTP_PAYMENT_REQUIRED:
-            error += (u'\nYou\'ll need to buy some more credits to perform'
-                      u' the chosen action')
-            return error
-
-    return "Invalid %s structure:\n\n%s" % (resource_type, resource)
-
-
 def assign_dir(path):
     """Silently checks the path for existence or creates it.
 
@@ -552,7 +475,7 @@ class BigML(object):
 
     """
     def __init__(self, username=None, api_key=None, dev_mode=False,
-                 debug=False, set_locale=False, storage=None):
+                 debug=False, set_locale=False, storage=None, domain=None):
         """Initializes the BigML API.
 
         If left unspecified, `username` and `api_key` will default to the
@@ -566,6 +489,11 @@ class BigML(object):
         If storage is set to a directory name, the resources obtained in
         CRU operations will be stored in the given directory.
 
+        If domain is set, the api will point to the specified domain. Default
+        will be the one in the environment variable `BIGML_DOMAIN` or
+        `bigml.io` if missing. The expected domain argument is a string or a
+        Domain object. See Domain class for details.
+
         """
 
         logging_level = logging.ERROR
@@ -592,24 +520,7 @@ class BigML(object):
         self.auth = "?username=%s;api_key=%s;" % (username, api_key)
         self.dev_mode = dev_mode
 
-        if dev_mode:
-            self.url = BIGML_DEV_URL
-            self.prediction_url = BIGML_DEV_URL
-        else:
-            self.url = BIGML_URL
-            self.prediction_url = BIGML_PREDICTION_URL
-
-        # Base Resource URLs
-        self.source_url = self.url + SOURCE_PATH
-        self.dataset_url = self.url + DATASET_PATH
-        self.model_url = self.url + MODEL_PATH
-        self.prediction_url = self.prediction_url + PREDICTION_PATH
-        self.evaluation_url = self.url + EVALUATION_PATH
-        self.ensemble_url = self.url + ENSEMBLE_PATH
-        self.batch_prediction_url = self.url + BATCH_PREDICTION_PATH
-        self.cluster_url = self.url + CLUSTER_PATH
-        self.centroid_url = self.url + CENTROID_PATH
-        self.batch_centroid_url = self.url + BATCH_CENTROID_PATH
+        self._set_api_urls(dev_mode=dev_mode, domain=domain)
 
         if set_locale:
             locale.setlocale(locale.LC_ALL, DEFAULT_LOCALE)
@@ -626,7 +537,45 @@ class BigML(object):
             'centroid': self.get_centroid,
             'batchcentroid': self.get_batch_centroid}
 
-    def _create(self, url, body, verify=VERIFY):
+    def _set_api_urls(self, dev_mode=False, domain=None):
+        """Sets the urls that point to the REST api methods for each resource
+
+        """
+        if domain is None:
+            domain = Domain()
+        elif isinstance(domain, basestring):
+            domain = Domain(domain=domain)
+        elif not isinstance(domain, Domain):
+            raise ValueError("The domain must be set using a Domain object.")
+        # Setting the general and prediction domain options
+        self.general_domain = domain.general_domain
+        self.prediction_domain = domain.prediction_domain
+        self.prediction_protocol = domain.prediction_protocol
+        self.verify = domain.verify
+        self.verify_prediction = domain.verify_prediction
+        if dev_mode:
+            self.url = BIGML_DEV_URL % (DEFAULT_PROTOCOL, self.general_domain)
+            self.prediction_url = BIGML_DEV_URL % (DEFAULT_PROTOCOL,
+                                                   self.general_domain)
+        else:
+            self.url = BIGML_URL % (DEFAULT_PROTOCOL, self.general_domain)
+            self.prediction_url = BIGML_URL % (
+                self.prediction_protocol, self.prediction_domain)
+
+        # Base Resource URLs
+        self.source_url = self.url + SOURCE_PATH
+        self.dataset_url = self.url + DATASET_PATH
+        self.model_url = self.url + MODEL_PATH
+        self.prediction_url = self.prediction_url + PREDICTION_PATH
+        self.evaluation_url = self.url + EVALUATION_PATH
+        self.ensemble_url = self.url + ENSEMBLE_PATH
+        self.batch_prediction_url = self.url + BATCH_PREDICTION_PATH
+        self.cluster_url = self.url + CLUSTER_PATH
+        self.centroid_url = self.url + CENTROID_PATH
+        self.batch_centroid_url = self.url + BATCH_CENTROID_PATH
+
+
+    def _create(self, url, body, verify=None):
         """Creates a new remote resource.
 
         Posts `body` in JSON to `url` to create a new remote resource.
@@ -652,6 +601,8 @@ class BigML(object):
         # return a HTTP_ACCEPTED (202) while the model or ensemble is being
         # downloaded.
         code = HTTP_ACCEPTED
+        if verify is None:
+            verify = self.verify
         while code == HTTP_ACCEPTED:
             try:
                 response = requests.post(url + self.auth,
@@ -671,7 +622,7 @@ class BigML(object):
                               HTTP_NOT_FOUND,
                               HTTP_TOO_MANY_REQUESTS]:
                     error = json.loads(response.content, 'utf-8')
-                    LOGGER.error(error_message(error, method='create'))
+                    LOGGER.error(self.error_message(error, method='create'))
                 elif code != HTTP_ACCEPTED:
                     LOGGER.error("Unexpected error (%s)" % code)
                     code = HTTP_INTERNAL_SERVER_ERROR
@@ -720,7 +671,7 @@ class BigML(object):
         try:
             response = requests.get(url + auth + query_string,
                                     headers=ACCEPT_JSON,
-                                    verify=VERIFY)
+                                    verify=self.verify)
 
             code = response.status_code
 
@@ -733,7 +684,7 @@ class BigML(object):
                           HTTP_NOT_FOUND,
                           HTTP_TOO_MANY_REQUESTS]:
                 error = json.loads(response.content, 'utf-8')
-                LOGGER.error(error_message(error, method='get'))
+                LOGGER.error(self.error_message(error, method='get'))
             else:
                 LOGGER.error("Unexpected error (%s)" % code)
                 code = HTTP_INTERNAL_SERVER_ERROR
@@ -783,7 +734,7 @@ class BigML(object):
         try:
 
             response = requests.get(url + self.auth + query_string,
-                                    headers=ACCEPT_JSON, verify=VERIFY)
+                                    headers=ACCEPT_JSON, verify=self.verify)
             code = response.status_code
 
             if code == HTTP_OK:
@@ -842,7 +793,7 @@ class BigML(object):
         try:
             response = requests.put(url + self.auth,
                                     headers=SEND_JSON,
-                                    data=body, verify=VERIFY)
+                                    data=body, verify=self.verify)
 
             code = response.status_code
 
@@ -855,7 +806,7 @@ class BigML(object):
                           HTTP_METHOD_NOT_ALLOWED,
                           HTTP_TOO_MANY_REQUESTS]:
                 error = json.loads(response.content, 'utf-8')
-                LOGGER.error(error_message(error, method='update'))
+                LOGGER.error(self.error_message(error, method='update'))
             else:
                 LOGGER.error("Unexpected error (%s)" % code)
                 code = HTTP_INTERNAL_SERVER_ERROR
@@ -887,7 +838,7 @@ class BigML(object):
                 "message": "The resource couldn't be deleted"}}
 
         try:
-            response = requests.delete(url + self.auth, verify=VERIFY)
+            response = requests.delete(url + self.auth, verify=self.verify)
             code = response.status_code
 
             if code == HTTP_NO_CONTENT:
@@ -897,7 +848,7 @@ class BigML(object):
                           HTTP_NOT_FOUND,
                           HTTP_TOO_MANY_REQUESTS]:
                 error = json.loads(response.content, 'utf-8')
-                LOGGER.error(error_message(error, method='delete'))
+                LOGGER.error(self.error_message(error, method='delete'))
             else:
                 LOGGER.error("Unexpected error (%s)" % code)
                 code = HTTP_INTERNAL_SERVER_ERROR
@@ -924,7 +875,7 @@ class BigML(object):
         file_object = None
 
         response = requests.get(url + self.auth,
-                                verify=VERIFY, stream=True)
+                                verify=self.verify, stream=True)
         code = response.status_code
 
         if code == HTTP_OK:
@@ -1000,6 +951,59 @@ class BigML(object):
             LOGGER.error("The resource couldn't be created: %s" %
                          resource['error'])
 
+    def error_message(self, resource, resource_type='resource', method=None):
+        """Error message for each type of resource
+
+        """
+        error = None
+        error_info = None
+        if isinstance(resource, dict):
+            if 'error' in resource:
+                error_info = resource['error']
+            elif ('code' in resource
+                  and 'status' in resource):
+                error_info = resource
+        if error_info is not None and 'code' in error_info:
+            code = error_info['code']
+            if ('status' in error_info and
+                    'message' in error_info['status']):
+                error = error_info['status']['message']
+                extra = error_info['status'].get('extra', None)
+                if extra is not None:
+                    error += ": %s" % extra
+            if code == HTTP_NOT_FOUND and method == 'get':
+                alternate_message = ''
+                if self.general_domain != DEFAULT_DOMAIN:
+                    alternate_message = (
+                        u'- The %s was not created in %s.\n' % (
+                            resource_type, self.general_domain))
+                error += (
+                    u'\nCouldn\'t find a %s matching the given'
+                    u' id. The most probable causes are:\n\n%s'
+                    u'- A typo in the %s\'s id.\n'
+                    u'- The %s id cannot be accessed with your credentials.\n'
+                    u'\nDouble-check your %s and'
+                    u' credentials info and retry.' % (
+                        resource_type, alternate_message, resource_type,
+                        resource_type, resource_type))
+                return error
+            if code == HTTP_UNAUTHORIZED:
+                error += u'\nDouble-check your credentials, please.'
+                return error
+            if code == HTTP_BAD_REQUEST:
+                error += u'\nDouble-check the arguments for the call, please.'
+                return error
+            if code == HTTP_TOO_MANY_REQUESTS:
+                error += (u'\nToo many requests. Please stop '
+                          u' requests for a while before resuming.')
+                return error
+            elif code == HTTP_PAYMENT_REQUIRED:
+                error += (u'\nYou\'ll need to buy some more credits to perform'
+                          u' the chosen action')
+                return error
+
+        return "Invalid %s structure:\n\n%s" % (resource_type, resource)
+
     ##########################################################################
     #
     # Utils
@@ -1211,7 +1215,7 @@ class BigML(object):
         try:
             response = requests.post(self.source_url + self.auth,
                                      files=files,
-                                     data=create_args, verify=VERIFY)
+                                     data=create_args, verify=self.verify)
 
             code = response.status_code
             if code == HTTP_CREATED:
@@ -1367,7 +1371,7 @@ class BigML(object):
                         HTTP_TOO_MANY_REQUESTS]:
                 content = exception.read()
                 error = json.loads(content, 'utf-8')
-                LOGGER.error(error_message(error, method='create'))
+                LOGGER.error(self.error_message(error, method='create'))
             else:
                 LOGGER.error("Unexpected error (%s)" % code)
                 code = HTTP_INTERNAL_SERVER_ERROR
@@ -1461,12 +1465,20 @@ class BigML(object):
     # https://bigml.com/developers/datasets
     #
     ##########################################################################
-    def create_dataset(self, source_or_datasets, args=None,
+    def create_dataset(self, origin_resource, args=None,
                        wait_time=3, retries=10):
         """Creates a remote dataset.
 
-        Uses remote `source` or `dataset` to create a new dataset using the
+        Uses a remote resource to create a new dataset using the
         arguments in `args`.
+        The allowed remote resources can be:
+            - source
+            - dataset
+            - list of datasets
+            - cluster
+        In the case of using cluster id as origin_resources, a centroid must
+        also be provided in the args argument. The first centroid is used
+        otherwise.
         If `wait_time` is higher than 0 then the dataset creation
         request is not sent until the `source` has been created successfuly.
 
@@ -1475,14 +1487,16 @@ class BigML(object):
         if args is not None:
             create_args.update(args)
 
-        if isinstance(source_or_datasets, list):
+        if isinstance(origin_resource, list):
+            # mutidatasets
             create_args = self._set_create_from_datasets_args(
                 source_or_datasets, args=create_args, wait_time=wait_time,
                 retries=retries, key="origin_datasets")
         else:
-            resource_type = get_resource_type(source_or_datasets)
+            # dataset from source
+            resource_type = get_resource_type(origin_resource)
             if resource_type == SOURCE_PATH:
-                source_id = get_source_id(source_or_datasets)
+                source_id = get_source_id(origin_resource)
                 if source_id:
                     check_resource(source_id, self.get_source,
                                    wait_time=wait_time,
@@ -1490,13 +1504,33 @@ class BigML(object):
                                    raise_on_error=True)
                     create_args.update({
                         "source": source_id})
+            # dataset from dataset
             elif resource_type == DATASET_PATH:
                 create_args = self._set_create_from_datasets_args(
-                    source_or_datasets, args=create_args, wait_time=wait_time,
+                    origin_resource, args=create_args, wait_time=wait_time,
                     retries=retries, key="origin_dataset")
+            # dataset from cluster and centroid
+            elif resource_type == CLUSTER_PATH:
+                cluster_id = get_cluster_id(origin_resource)
+                cluster = check_resource(cluster_id, self.get_cluster,
+                                         wait_time=wait_time,
+                                         retries=retries,
+                                         raise_on_error=True)
+                if not 'centroid' in create_args:
+                    try:
+                        centroid = cluster['object'][
+                            'cluster_datasets_ids'].keys()[0]
+                        create_args.update({'centroid': centroid})
+                    except KeyError:
+                        raise KeyError("Failed to generate the dataset. A "
+                                       "centroid id is needed in the args "
+                                       "argument to generate a dataset from "
+                                       "a cluster.")
+                create_args.update({'cluster': cluster_id})
             else:
-                raise Exception("A source, dataset or list of dataset ids"
-                                " are needed to create a"
+                raise Exception("A source, dataset, list of dataset ids"
+                                " or cluster id plus centroid id are needed"
+                                " to create a"
                                 " dataset. %s found." % resource_type)
 
         body = json.dumps(create_args)
@@ -1707,7 +1741,7 @@ class BigML(object):
 
         body = json.dumps(create_args)
         return self._create(self.prediction_url, body,
-                            verify=VERIFY_PREDICTION_SERVER)
+                            verify=self.verify_prediction)
 
     def get_prediction(self, prediction):
         """Retrieves a prediction.
@@ -2077,7 +2111,7 @@ class BigML(object):
 
         body = json.dumps(create_args)
         return self._create(self.centroid_url, body,
-                            verify=VERIFY)
+                            verify=self.verify)
 
     def get_centroid(self, centroid):
         """Retrieves a centroid.
